-- -------------------------------------------------------------------------------------
gma.feedback("disguise Cue Creator for GrandMA2 by David Butler")
gma.feedback("Version 1.0 - 11/09/2019")
-- -------------------------------------------------------------------------------------

-- -------------------------------------------------------------------------------------
-- ABOUT
-- -------------------------------------------------------------------------------------
-- This plugin is designed to automatically create cues in an MA sequence to control a 
-- diguise timeline fixture using an exported cue table from a disguise track.
-- The plugin works by extracting all cues from the disguise cue table which are 
-- correctly formatted for DMX recall (in the format XX.YY.ZZ), and then searching
-- a specified MA sequence for cues with matching numbers. Matching cues will have an 
-- additional part cue added with a cue number programmed onto the disguise timeline 
-- fixture, and a label taken from the disguise cue name.

-- -------------------------------------------------------------------------------------
-- INSTRUCTIONS
-- -------------------------------------------------------------------------------------
--  - Patch a disguise timeline control fixture on MA. (This plugin is 
--    pre-configured for the factory MA personality).
--  - Program a sequence on MA with all control data required (lighting, etc.) but 
--    do not program any data for the disguise timeline fixture.
--  - Do not program any control data in the part number to be used for disguise 
--    cues (part 99 by default, but configurable in settings below).
--  - Program your cues in disguise in the format XX.YY.ZZ. E.g. Cue 102.5 would 
--    be 01.02.50. Use the same cue numbers as in your MA sequence.
--  - Export a cue table from a disguise track. Copy the file to the MA root folder.
--    This is 'C:\ProgramData\MA Lighting Technologies\grandma\gma2_V_3.7' for OnPC. 
--    For a console you will need to access via FTP or file sharing.
--  - Run the plugin!
--  - If you need to update the programming. Reimport a new cue table from disguise 
--    and run the plugin again. 
--  - You will be offered the option to remove all existing programming for Part 
--    99 on every cue. Do this only if updating form the same disguise track. 
--  - If you are importing cues from a different disguise track, click 'Cancel'
--    and the new data will be merged into the existing sequence.
--  - To remove all data programmed by the plugin, run the command 'Delete Sequence 
--    [sequence number] Cue Thru Part 99 /dv'.

-- -------------------------------------------------------------------------------------
-- SETTINGS
-- -------------------------------------------------------------------------------------

-- The part number which will be used for video cues. 
-- Only use a number which is not already in use in your cuelist.
local VIDEO_CUE_PART_NUMBER = 99                

-- Names for the 'CueXX', 'CueYY' and 'CueZZ' attributes in your Disguise timeline 
-- fixture personality. This plugin is pre-configured for the factory MA personality.
-- If you are using the timeline personality exported from disguise you will need to 
-- change these values.
local DISGUISE_CUEXX_ATTRIBUTE_NAME = "Cue" 
local DISGUISE_CUEYY_ATTRIBUTE_NAME = "Cue2"
local DISGUISE_CUEZZ_ATTRIBUTE_NAME = "Cue3"    

-- If true, video part cue names will have a custom text prefix.
local SHOULD_PREFIX_VIDEO_CUE_NAME = true       
local VIDEO_CUE_NAME_PREFIX = "VIDEO:"

-- If true, video part cues will be set to a custom color. RGB values are 0-100 range.
local SHOULD_COLOR_VIDEO_CUES = true
local VIDEO_CUE_COLOR_RED = 0
local VIDEO_CUE_COLOR_GREEN = 50
local VIDEO_CUE_COLOR_BLUE = 0

-- -------------------------------------------------------------------------------------
-- -------------------------------------------------------------------------------------
-- -------------------------------------------------------------------------------------
-- -------------------------------------------------------------------------------------
-- -------------------------------------------------------------------------------------
-- DO NOT ADJUST ANYTHING BELOW THIS LINE
-- -------------------------------------------------------------------------------------
-- -------------------------------------------------------------------------------------
-- -------------------------------------------------------------------------------------
-- -------------------------------------------------------------------------------------
-- -------------------------------------------------------------------------------------

-- Constants
local DEFAULT_DISGUISE_CUE_TABLE_FILENAME = "disguise_cue_table.txt"
local VARNAME_DISGUISE_CUE_TABLE_FILENAME = "___DCC_CUE_TABLE_FILENAME"
local DEFAULT_SEQUENCE_NUMBER = 1
local VARNAME_SEQUENCE_NUMBER = "___DCC_SEQUENCE_NUMBER"
local DEFAULT_DISGUISE_FIXTURE_NUMBER = 1
local VARNAME_DISGUISE_FIXTURE_NUMBER = "___DCC_DISGUISE_FIXTURE_NUMBER"

-- Load the default values from user vars
local disguise_cue_table_filename = gma.user.getvar(VARNAME_DISGUISE_CUE_TABLE_FILENAME)
if disguise_cue_table_filename == nil then
    disguise_cue_table_filename = DEFAULT_DISGUISE_CUE_TABLE_FILENAME
    gma.user.setvar(VARNAME_DISGUISE_CUE_TABLE_FILENAME, disguise_cue_table_filename)
end

local sequence_number = gma.user.getvar(VARNAME_SEQUENCE_NUMBER)
if sequence_number == nil then
    sequence_number = DEFAULT_SEQUENCE_NUMBER
    gma.user.setvar(VARNAME_SEQUENCE_NUMBER, sequence_number)
end

local disguise_fixture_number = gma.user.getvar(VARNAME_DISGUISE_FIXTURE_NUMBER)
if disguise_fixture_number == nil then
    disguise_fixture_number = DEFAULT_DISGUISE_FIXTURE_NUMBER
    gma.user.setvar(VARNAME_DISGUISE_FIXTURE_NUMBER, disguise_fixture_number)
end


function Start()

    gma.feedback("Starting disguise Cue Creator..")
    
    -- Ask the user for the variable inputs and test.
    
    disguise_cue_table_filename = gma.textinput("Disguise Cue Table Filename", disguise_cue_table_filename)
    if disguise_cue_table_filename == nil then
        gma.gui.msgbox("Disguise Cue Creator - Error", "Not a valid cue table filename")
        disguise_cue_table_filename = gma.user.getvar(VARNAME_DISGUISE_CUE_TABLE_FILENAME)
        gma.feedback("disguise Cue Creator ended with error")
        return
    end
    gma.user.setvar(VARNAME_DISGUISE_CUE_TABLE_FILENAME, disguise_cue_table_filename)
    
    local f = io.open(disguise_cue_table_filename, "r")
    if f == nil then
        gma.gui.msgbox("Disguise Cue Creator - Error", "Failed to open disguise cue table file '" .. disguise_cue_table_filename .. "'")
        io.close(f)
        gma.feedback("disguise Cue Creator ended with error")
        return
    end
    io.close(f)
    
    
    sequence_number = tonumber(gma.textinput("Sequence Number", sequence_number))
    if sequence_number == nil then
        gma.gui.msgbox("Disguise Cue Creator - Error", "Not a valid sequence number")
        sequence_number = gma.user.getvar(VARNAME_SEQUENCE_NUMBER)
        return
    end
    gma.user.setvar(VARNAME_SEQUENCE_NUMBER, sequence_number)
    
    local gma_sequence_handle = gma.show.getobj.handle("Sequence ".. sequence_number)
    if gma.show.getobj.verify (gma_sequence_handle) == false then
        gma.gui.msgbox("Disguise Cue Creator - Error", "Couldn't find a sequence with number " .. sequence_number)
        gma.feedback("disguise Cue Creator ended with error")
        return
    end
    
    disguise_fixture_number = tonumber(gma.textinput("Disguise Fixture Number", disguise_fixture_number))
    if disguise_fixture_number == nil then
        gma.gui.msgbox("Disguise Cue Creator - Error", "Not a valid fixture number")
        disguise_fixture_number = gma.user.getvar(VARNAME_DISGUISE_FIXTURE_NUMBER)
        gma.feedback("disguise Cue Creator ended with error")
        return
    end
    gma.user.setvar(VARNAME_DISGUISE_FIXTURE_NUMBER, disguise_fixture_number)
    
    local gma_fixture_handle = gma.show.getobj.handle("Fixture ".. disguise_fixture_number)
    if gma.show.getobj.verify (gma_fixture_handle) == false then
        gma.gui.msgbox("Disguise Cue Creator - Error", "Couldn't find a fixture with number " .. disguise_fixture_number)
        gma.feedback("disguise Cue Creator ended with error")
        return
    end
    
    local gma_fixture_attribute_handle = gma.show.getobj.handle("Fixture ".. disguise_fixture_number .. " Attribute \"" .. DISGUISE_CUEXX_ATTRIBUTE_NAME .. "\"")
    if gma.show.getobj.verify (gma_fixture_attribute_handle) == false then
        gma.gui.msgbox("Disguise Cue Creator - Error", "Fixture " .. disguise_fixture_number .. " does not have attribute named '" 
            .. DISGUISE_CUEXX_ATTRIBUTE_NAME .. "'")
        gma.feedback("disguise Cue Creator ended with error")
        return
    end
    
    gma_fixture_attribute_handle = gma.show.getobj.handle("Fixture ".. disguise_fixture_number .. " Attribute \"" .. DISGUISE_CUEYY_ATTRIBUTE_NAME .. "\"")
    if gma.show.getobj.verify (gma_fixture_attribute_handle) == false then
        gma.gui.msgbox("Disguise Cue Creator - Error", "Fixture " .. disguise_fixture_number .. " does not have attribute named '" 
            .. DISGUISE_CUEYY_ATTRIBUTE_NAME .. "'")
        gma.feedback("disguise Cue Creator ended with error")
        return
    end
    
    gma_fixture_attribute_handle = gma.show.getobj.handle("Fixture ".. disguise_fixture_number .. " Attribute \"" .. DISGUISE_CUEZZ_ATTRIBUTE_NAME .. "\"")
    if gma.show.getobj.verify (gma_fixture_attribute_handle) == false then
        gma.gui.msgbox("Disguise Cue Creator - Error", "Fixture " .. disguise_fixture_number .. " does not have attribute named '" 
            .. DISGUISE_CUEZZ_ATTRIBUTE_NAME .. "'")
        gma.feedback("disguise Cue Creator ended with error")
        return
    end
    
    -- Check for existing data
    
    local gma_existing_parts = gma.show.getobj.handle("Sequence ".. sequence_number .. " Cue Thru Part " .. VIDEO_CUE_PART_NUMBER)
    
    if gma.show.getobj.verify(gma_existing_parts) then
    
        local should_delete = gma.gui.confirm("Disguise Cue Creator", "Found existing data at Part " .. VIDEO_CUE_PART_NUMBER .. " for Sequence " 
        .. sequence_number ..". Do you want to delete this part for all cues?")
                            
        if should_delete then
            gma.cmd("Delete Sequence " .. sequence_number .. " Cue Thru Part " ..  VIDEO_CUE_PART_NUMBER .. "/dv")
        else
            local should_continue = gma.gui.confirm("Disguise Cue Creator", "Existing Part data will not be deleted. \nDo you want to continue and merge new cue data from the disguise cue table?")
            
            if should_continue == false then
                gma.gui.msgbox("Disguise Cue Creator - Error", "Aborting plugin. No sequence data has been changed.")
                return
            end
        end
    
    end
    
    -- Main loop
    
    gma.cmd("ClearAll")

    gma.feedback("Reading disguise cue data from '" .. disguise_cue_table_filename .. "'")
    
    local line_index = -2
    local cue_count = 0
    for line in io.lines(disguise_cue_table_filename) do

        if line_index >= 0 then

            local cue_info = {}
            for e in string.gmatch(line, "[^\t]+") do
                cue_info[#cue_info + 1] = e
            end

            local cue_number_text = string.match(cue_info[2], "CUE %d%d?%.%d%d?%.%d%d?")
            
            if cue_number_text ~= nil then
                
                cue_count = cue_count + 1
                
                local cue_number_elements = {}
                for n in string.gmatch(string.sub(cue_number_text, 5), "[^%.]+") do
                    cue_number_elements[#cue_number_elements + 1] = n
                end
                
                local cue_number = (tonumber(cue_number_elements[1]) * 100) + tonumber(cue_number_elements[2])
                
                if string.len(cue_number_elements[3]) == 2 then
                    cue_number = cue_number + (tonumber(cue_number_elements[3]) * 0.01)
                else
                    cue_number = cue_number + (tonumber(cue_number_elements[3]) * 0.1)
                end
                
                local cue_number_formatted = string.format(cue_number, "%.2f")
                local cue_name = cue_info[3]
                
                gma.feedback("Found disguise cue " .. cue_number_formatted .. " - " .. cue_name)

                local gma_cue_handle = gma.show.getobj.handle("Sequence ".. sequence_number .. " Cue " .. cue_number_formatted)
                    
                if gma.show.getobj.verify (gma_cue_handle) then
                        gma.feedback("Found matching MA cue " .. cue_number_formatted)                  
                    
                gma.cmd("Fixture " .. disguise_fixture_number .. " Attribute \"" .. DISGUISE_CUEXX_ATTRIBUTE_NAME .. "\" At " 
                    .. (tonumber(cue_number_elements[1]) / 255.) * 100.)
                gma.cmd("Fixture " .. disguise_fixture_number .. " Attribute \"" .. DISGUISE_CUEYY_ATTRIBUTE_NAME .. "\" At " 
                    .. (tonumber(cue_number_elements[2]) / 255.) * 100.)
                gma.cmd("Fixture " .. disguise_fixture_number .. " Attribute \"" .. DISGUISE_CUEZZ_ATTRIBUTE_NAME .. "\" At " 
                    .. (tonumber(cue_number_elements[3]) / 255.) * 100.)
                
                local video_cue_part_name
                if SHOULD_PREFIX_VIDEO_CUE_NAME then
                    video_cue_part_name = VIDEO_CUE_NAME_PREFIX .. " " .. cue_name
                else
                    video_cue_part_name = cue_name
                end
                
                gma.cmd("Store Sequence ".. sequence_number .. " Cue " .. cue_number_formatted 
                    .." Part " .. VIDEO_CUE_PART_NUMBER .. " \"" .. video_cue_part_name .. "\" Fade 0 /tracking=True /nc /m")
                
                if SHOULD_COLOR_VIDEO_CUES then
                    gma.cmd("Appearance Cue " .. cue_number_formatted .. " Part " .. VIDEO_CUE_PART_NUMBER
                        .. " /red=".. VIDEO_CUE_COLOR_RED .." /green=" .. VIDEO_CUE_COLOR_GREEN .. " /blue=" .. VIDEO_CUE_COLOR_BLUE)
                end
                    
            else
                gma.feedback("No matching MA cue " .. cue_number_formatted)
            end
                
            end

        end
        
        line_index = line_index + 1
    end
    
    if cue_count == 0 then
        gma.gui.msgbox("Disguise Cue Creator - Error", "Didn't find any usable cues in the disguise cue table.")
    end
    
    gma.cmd("ClearAll")

    gma.feedback("disguise Cue Creator complete")
end

return Start
